/* par_log_act.c */
/*
   Copyright (C) 2005 John W. Jordan
   All Rights Reserved
*/
/*
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or 
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */  
/* Action routines for log.dan. */
/* Builds a logical tree. */
#include <stdio.h>
#include "par.h"
#include "act.h"
void Push (STACK * stack, NODE * node);
NODE *Pop (STACK * stack);
NODE *mk_node (int type, char *name);
void pr_tree (NODE * node);
void pr_stack (STACK *stack);
void pr_node (NODE * node);
char *type_name (int type);
void *stk_operand;
void *stk_operator;
/* --------------------------------------------------------- */
/* Called from main to initialize actions. */
void par_log_act ( PAR *par)
{
    stk_operator = (void *) CreateStack (10, 5);  
    stk_operand = (void *) CreateStack (20, 20);    
} 
/* --------------------------------------------------------- */
NODE *
top ()
{
NODE *x;

  x = Pop (stk_operand);
  Push (stk_operand, x);
  return x;
}

/* --------------------------------------------------------- */
/* Record a binary operation. */
int
par_a_bop (PAR * par)
{
  NODE *node;

  node = mk_node (par->type,strdup (par->symbol) );
  node->op_ary = OPR_BINARY;
  Push (stk_operator, node);
}
/* --------------------------------------------------------- */
/* Record a uniary operation. */
int
par_a_uop (PAR * par)
{
  NODE *node;

  node = mk_node (par->type, strdup (par->symbol));  
  node->op_ary = OPR_UNITARY;
  Push (stk_operator, node);
}

/* --------------------------------------------------------- */
/* Build a bottom up tree of operations and operands. */
int
par_a_tree (PAR * par)
{  
  NODE *op;

  op = Pop (stk_operator);
  
  switch (op->op_ary)
    {
    case OPR_BINARY:
      op->right = Pop (stk_operand);
      op->left = Pop (stk_operand);
      break;
    case OPR_UNITARY:
      op->left = Pop (stk_operand);     
      break;
    }
  Push (stk_operand, op);
}

/* --------------------------------------------------------- */
int
par_a_iden (PAR * par)
{
  NODE *node;

  node = mk_node ('I', strdup (par->symbol));
  Push (stk_operand, node);
}

/* --------------------------------------------------------- */
NODE *
mk_node (int type, char *name)
{
  NODE *node;
  static int id = 1;

  node = (NODE *) malloc (sizeof (NODE));
  node->id = id++;
  node->type = type;
  node->name = name;
  node->left = 0;
  node->right = 0;
  node->op_ary = 0;
  return node;
}

/* --------------------------------------------------------- */
void
pr_node (NODE * node)
{
  if (node)
    {
      printf ("node id %2d type %c ", node->id, node->type);

      if (node->left)
	printf ("left %2d ", node->left->id);
      else
	printf ("left %2d ", 0);

      if (node->right)
	printf ("right %2d ", node->right->id);
      else
	printf ("right %2d ", 0);

      printf ("name %s ", node->name);
      printf ("\n");
    }
}

/* --------------------------------------------------------- */
void
set_tree_id (NODE * node)
{
  static int count = 1;

  if (node == 0)
    return;
  node->id = count++;
  set_tree_id (node->left);
  set_tree_id (node->right);
}

/* --------------------------------------------------------- */
void
pr_tree_prefix (NODE * node)
{
  if (node == 0)
    return;
  pr_node (node);
  pr_tree_prefix (node->left);
  pr_tree_prefix (node->right);
}

/* --------------------------------------------------------- */
void
pr_tree_infix (NODE * node)
{
  if (node == 0)
    return;
  pr_tree_infix(node->left);
  pr_node (node);
  pr_tree_infix (node->right);
}

/* --------------------------------------------------------- */
void
pr_tree_postfix (NODE * node)
{
  if (node == 0)
    return;
  pr_tree_postfix (node->left);
  pr_tree_postfix (node->right);
  pr_node (node);
}