/* par_lex_input.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved

  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* This input routine uses a table to determine char. types
   instead of the common C macros. Its a little neater but may 
   require that the table be changed to fit the application.
   However, this default table should be good for most applications.
   Of course, the macros may be used instead.
*/
 /* 
     Assign type to character. 
     Same as these std. functions: 
     isalpha lex_t = 'L'
     isdigit lex_t = 'D'
     isspace lex_t = 'S'
     Except newline is not considered space.
     Punctuation characters type is equal to char. value.
     i.e. ispunct  lex_t = lex_c 
     !isprint lex_t = 'E' - this is an error condition as
     non printable char. should never appear int test input.
     This could occur if a non text file was input.
   */
#include <stdio.h>
#include <string.h>
#include "par.h"
/* ------------------------------------------- */
static const int debug = 0;
/* ------------------------------------------- */
static const unsigned char ctypes[128] = {
 0,  'E', 'E', 'E', 'E', 'E', 'E', 'E',
'E', 'S',  10, 'S', 'S', 'S', 'E', 'E',
'E', 'E', 'E', 'E', 'E', 'E', 'E', 'E',
'E', 'E', 'E', 'E', 'E', 'E', 'E', 'E',
'S', '!', '"', '#', '$', '%', '&', '\'',
'(', ')', '*', '+', ',', '-', '.', '/',
'D', 'D', 'D', 'D', 'D', 'D', 'D', 'D',
'D', 'D', ':', ';', '<', '=', '>', '?',
'@', 'L', 'L', 'L', 'L', 'L', 'L', 'L',
'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L',
'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L',
'L', 'L', 'L', '[', '\\',']', '^', '_',
'`', 'L', 'L', 'L', 'L', 'L', 'L', 'L',
'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L',
'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L',
'L', 'L', 'L', '{', '|', '}', '~', 'E'
};

/* --------------------------------------------- */
int
par_lex_input (PAR * par)
{
   if (par->buffer_input)
    {
      if (par->buffer)
        {
          if (par->buf_pos == par->buf_len)
            par->lex_c = -1;
          else
            par->lex_c = par->buffer[(par->buf_pos)++];
        }
      else
        {
          fprintf (stderr, 
            "PAR_LEX_INPUT: Input buffer ptr. is null\n");
          exit (1);
        }
    }
  else
    par->lex_c = fgetc (par->fd);
    
  /* End of file - use 0 */
  if (par->lex_c == (-1))
    {
      par->lex_c = 0;
      par->lex_t = 0;
    } 

  par->lex_t = ctypes[par->lex_c];

  /* Newline - not considered as a space char. */

  if (par->lex_c == '\n')
    par->lineno++; 
      
  if (par->lex_t == 'E')
       {
         fprintf (stderr, "LEX INPUT: non printable character [dec. %d hex. %x]\n", 
                  par->lex_c, par->lex_c);
         exit (1);
       }

  if (par->print_input)
    {
      if (par->lex_c == '\n')
	printf ("\n%4d ", par->lineno);
      else
	printf ("%c", par->lex_c);
    }

     if(debug) printf ("INPUT: lex_c [%c %d] lex_t [%c %d] \n", 
               par->lex_c,  par->lex_c, par->lex_t, par->lex_t);
     
  return 1;
}
