/* par_dan_act.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* 
   Dante action routines.
*/
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include "par.h"
#include "dan_act.h"
#include "par_stack.h"
#include "par_sym.h"
/* --------------------------------------------------------- */
void pr_stacks (PAR * par, char *label);
NODE *mk_node (PAR * par);
NODE *par_find_rule (PAR * par, char *rname);
void par_add_rule (PAR * par, NODE * rule);
/* --------------------------------------------------------- */
static const int debug = 0;
/* ----------------------------------------------------------*/
/* INITIALIZE ACTION ROUTINES */
int
par_dan_act (PAR * par)
{
  NODE *node;

  par->stk_oper = (void *) CreateStack (10, 5);
  par->stk_opnd = (void *) CreateStack (20, 20);
  par->rwtbl = (void *) CreateStack (50, 100);
  par->stk_literal = (void *) CreateStack (20, 20);

  /* 
     Add the start (network)  node.
     Network node of type N is top of the tree.
     Static code is added to this node. 
   */
  node = (NODE *) mk_node (par);

  /* Node name is reset in action par_a_rname. */
  node->name = "N";
  node->type = 'N';

  /* Network node is top of rule list. */
  par_add_rule (par, node);
  return 1;
}

/* ============================ ACTIONS =================== */
/* Action routine for a rule name. */
int
par_a_rule (PAR * par)
{
  NODE *node;
  NODE *net_node;
  char *rname;

  rname = strdup (par->symbol);
  if (par_find_rule (par, rname))
    {
      fprintf (stderr, "ERROR: Duplicate rule name [%s]\n", rname);
      par->error_flag = 1;
      exit (1);
    }
  /* Set pname here. */
  if (par->pname == 0)
    {
      par->pname = rname;
      /* Set the name of the lexer. */
      par->lname = par->pname;
      /* Set network node. */
      net_node = (NODE *) par->top_rule;
      net_node->name = rname;
      if (debug)
	printf ("Set pname %s\n", par->pname);
    }
  /* Rule definition node. */
  node = mk_node (par);
  node->name = rname;
  node->type = 'D';
  Push (par->stk_opnd, node);
}

/* --------------------------------------------------------- */
/* Define a quoted reserved word. */
int
par_a_res (PAR * par)
{
  NODE *node;
  void *sym;

  node = mk_node (par);
  node->type = 'I';
  sym = (void *) sym_add (par->rwtbl, par->symbol);
  node->name = (char *) sym_name (sym);
  sym_set_res (sym, TRUE);
  node->sym = sym;
  Push (par->stk_opnd, node);
}

/* --------------------------------------------------------- */
int
par_a_iden (PAR * par)
{
  NODE *node;
  void *sym;
  int slot;

  node = mk_node (par);
  if (islower (par->symbol[0]))
    {
      node->type = 'R';
      node->name = strdup (par->symbol);
    }
  else
    {
      par_lower (par->symbol, par->symbol);
      node->type = 'I';
      sym = (void *) sym_add (par->rwtbl, par->symbol);
      node->name = (char *) sym_name (sym);
      sym_set_res (sym, FALSE);
      node->sym = sym;
    }
  Push (par->stk_opnd, node);
}

/* --------------------------------------------------------- */
/* Literal is same as iden except for node type. */
int
par_a_lit (PAR * par)
{
  NODE *node;
  void *sym;

  node = mk_node (par);
  node->type = 'L';
  sym = (void *) sym_add (par->stk_literal, par->symbol);
  node->name = (char *) sym_name (sym);
  sym_set_res (sym, TRUE);
  node->sym = sym;
  Push (par->stk_opnd, node);
}

/* --------------------------------------------------------- */
/* processing for the end of a rule. */
int
par_a_end (PAR * par)
{
  NODE *rule;
  NODE *expr;

  expr = Pop (par->stk_opnd);
  rule = Pop (par->stk_opnd);
  /* If rule is null this is an action rule. */
  if (rule == 0)
    {
      par_add_rule (par, expr);
      return;
    }
  /* Attach rule nodes to expr node. */
  rule->suc = expr;
  rule->nlink = expr;
  /* At the end of expr current symbol should be ;. */
  if (strcmp (par->symbol, ";") != 0)
    {
      fprintf (stderr, "ERROR:Rule [%s] did not end in semicolin ;\n",
	       rule->name);
    }
  /* Put all rule nodes on the rule list. */
  par_add_rule (par, rule);
}

/* ---------------------------------------- */
/* Find rule on rule list by name. */
NODE *
par_find_rule (PAR * par, char *rname)
{
  NODE *rule;

  rule = (NODE *) par->top_rule;
  while (rule = rule->rlink)
    if (strcmp (rname, rule->name) == 0)
      return rule;
  return 0;
}

/* ---------------------------------------- */
/* Add rule to rule list. */
void
par_add_rule (PAR * par, NODE * rule)
{
  NODE *top_rule;
  NODE *last_rule;

  top_rule = (NODE *) par->top_rule;
  last_rule = (NODE *) par->last_rule;
  if (top_rule == 0)
    {
      top_rule = rule;
      last_rule = rule;
    }
  else
    {
      last_rule->rlink = rule;
      rule->rlink = 0;
      last_rule = rule;
    }
  par->top_rule = top_rule;
  par->last_rule = last_rule;
}

/* ---------------------------------------- */
void
pr_stacks (PAR * par, char *label)
{
  printf ("---------------------------\n");
  printf ("%s:\n", label);
  pr_stack_nodes (par->stk_opnd, "STK: ");
}

/* ---------------------------------------- */
/* Purpose of the eof rule is to report on extraneous 
text left after Dante Completes.
*/
int
par_a_eof (PAR * par)
{
  if (par->type == 0)
    return 1;
  printf ("ERROR: Expected end of file.\n");
  printf ("ERROR: Unexpected text:[%s]\n", par->symbol);
  exit (1);
};
