/* par_dan_main.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "par.h"
#include "dan_act.h"

#define SYMBOL_LIMIT 1024

/* --------------------------------------------- */
void par_init (PAR * par, int slimit);
void par_gen_par (PAR * par);
void par_gen_main (PAR * par);
void par_gen_init (PAR * par);
void par_gen_file (PAR * par);
void par_gen_rwtbl (PAR * par);
void par_gen_mk (PAR * par);
void par_gen_code_net (PAR * par);
void par_gen_flex (PAR * par);
void par_dan_check (PAR * par);
static void rel();
/* --------------------------------------------- */
extern char *optarg;
extern int optind;
static const int debug = 0;
/* --------------------------------------------- */
int
main (int no_args, char *args[])
{
  PAR st_par;
  PAR *par = &st_par;
  char *filename;
  int c;
  int code = 300;

  int vflag = 0;		/* verboose, */
  int nflag = 0;		/* make file on nodes. */
  int lflag = 0;		/* make default lexer. */
  int hflag = 0;		/* make html table of nodes and actions. */
  int aflag = 0;		/* print lexer output. */
  int bflag = 0;		/* print input. */
  int iflag = 0;		/* generate interperter code. */

  while ((c = getopt (no_args, args, "vhnlabi")) != EOF)
    switch (c)
      {
      case 'v':
	vflag = 1;
	break;
      case 'h':
        rel();
	hflag = 1;
	break;
      case 'n':        
	nflag = 1;
	break;
      case 'l':
	lflag = 1;
	break;
      case 'a':
	aflag = 1;
	break;
      case 'b':
	bflag = 1;
	break;
      case 'i':
        rel();
	iflag = 1;
	break;
      case '?':
	exit (1);
	break;
      }

  if (args[optind] == 0)
    {
      printf ("USAGE:par [-v -n -l -h -i] file.\n");
      printf ("Input file was not specified.\n");
      exit (1);
    }

  if (access (args[optind], F_OK) == 0)
    {
      filename = args[optind];
    }
  else
    {
      filename = strdup (strcat (args[optind], ".dan"));
      if (access (filename, F_OK) != 0)
	{
	  fprintf (stderr, "ERROR: Cannot find input file %s\n", filename);
	  exit (1);
	}
    }

  /* Initialize par structure. */

  par_dan_init (par, SYMBOL_LIMIT);

  par->opt_verbose = vflag;
  par->opt_net_file = nflag;
  par->opt_html = hflag;
  par->opt_default_lexer = lflag;
  par->print_input = bflag;
  par->print_lexout = aflag;
  par->gen_intp = iflag;

  /* Init. action routines. */
  par_dan_act (par);
  /* parse input. */
  par_dan_file (par, filename);

  /* Resolve code references. */
  par_dan_cref (par);

  /* Set node numbers. */
  par_set_no (par);

  /* Check for some illegal language. */
  par_dan_check (par);

  /* Set type codes */
  code = sym_set_opr_table_codes (par->stk_literal, code);
  code = sym_set_res_table_codes (par->rwtbl, code);

  /* Generate code. */

  if (par->error_flag)
    {
      fprintf (stderr, "ERROR: No code generation because of error.\n");
      exit (1);
    }

  if (!par->sub_of)
    par_gen_rwtbl (par);

  par_gen_code_net (par);

  if (!par->gen_lexer)
    par_gen_error (par);

  /* Generate supporting procedures. */
  if (par->no_support == FALSE)
    {
      par_gen_par (par);
      par_gen_init (par);
      par_gen_main (par);
      par_gen_file (par);
      par_gen_act (par);
      par_gen_mk (par);
    }

  /* lexers */
  if (par->gen_flex)
    par_gen_flex (par);

  if (par->opt_default_lexer)
    par_gen_lex (par);

  /* Print tables. */
  if (par->opt_verbose)
    {
      sym_pr_symtbl (par->rwtbl);
      printf ("LITERALS:\n");
      sym_pr_stack (par->stk_literal);
    }
    
    if (par->opt_net_file)
     pr_nodes (par);

  return 0;
}
/* --------------------------------------------- */
static void rel()
{
  fprintf (stderr, "Option not in this release.\n");
  exit (1);
 }
 
 