/* par_dan_dir.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* Action routines for directives. */

#include "par.h"
#include "dan_act.h"
#include "par_sym.h"

/* --------------------------------------------------------- */
/* DAN_FLEX_DEF - NAME PART. */
/* The name of a token which is going to have a flex definition.
*/
int
par_a_flex_name (PAR * par)
{
  par_lower (par->symbol, par->symbol);
  par->flex_name = strdup (par->symbol);
}

/* --------------------------------------------------------- */
/* DAN_LEX_DEF - DEF PART. */
/* The flex definition. */
/* These go on the lexdef list. */
int
par_a_flex_def (PAR * par)
{
  sym_add_flexdef (par->rwtbl, par->flex_name, par->symbol);
}

/* --------------------------------------------------------- */
/* DAN_GEN_CODE. */
void
par_a_gen_code (PAR * par)
{
  par->gen_code = TRUE;
}

/* --------------------------------------------------------- */
/* DAN_GEN_FLEX. */
void
par_a_gen_flex (PAR * par)
{
  par->gen_flex = TRUE;
}

/* --------------------------------------------------------- */
/* DAN_USE_FLEX. */
void
par_a_use_flex (PAR * par)
{
  par->gen_flex = TRUE;
}

/* --------------------------------------------------------- */
/* DAN_GEN_INTP. */
void
par_a_gen_intp (PAR * par)
{
  par->gen_intp = TRUE;
}

/* --------------------------------------------------------- */
/* DAN_RESERVED - Specify reservered word. */
void
par_a_set_res (PAR * par)
{
  SYM *sym;

  par_lower (par->symbol, par->symbol);
  if ((sym = (SYM *) sym_find_symbol (par->rwtbl, par->symbol)) != 0)
    {
      sym->reserved = 1;
    }
  else
    {
      printf ("ERROR: For directive [dan_res]:\n");
      printf ("ERROR: :Name not found in symbol table:[%s]\n", par->symbol);
    }
}

/* --------------------------------------------------------- */
/* Generate only a lexer procedure. */
int
par_a_lex_fun (PAR * par)
{
  par->gen_lexer = TRUE;
  par->no_support = TRUE;
}

/* --------------------------------------------------------- */
/* Buffer input. */
int
par_a_buf_input (PAR * par)
{
  par->gen_buf_in = TRUE;
}

/* --------------------------------------------------------- */
int
par_a_gen_prod (PAR * par)
{
  par->gen_code = FALSE;
  par->gen_intp = FALSE;
}

/* --------------------------------------------------------- */
/*  */
int
par_a_no_support (PAR * par)
{
  par->no_support = TRUE;
}

/* --------------------------------------------------------- */
/* Set a internal type code. */
int
par_a_tname (PAR * par)
{
  par_lower (par->symbol, par->symbol);
  par->sym_name = strdup (par->symbol);
}

/* --------------------------------------------------------- */
/* Set a internal type code. */
int
par_a_tvalue (PAR * par)
{
  sym_set_type_by_name (par->rwtbl, par->sym_name, atoi (par->symbol));
}

/* --------------------------------------------------------- */
/* Compile a subroutine. */
int
par_a_sub_of (PAR * par)
{
  par->sub_of = 1;
  par->no_support = 1;
  par->lname = strdup (par->symbol);
}
